@@ -19,9 +19,12 @@
 
 #include <cassert>
 
+#include <QDebug>
 #include <QLineEdit>
 #include <QSpinBox>
 
+#include <libsigrokcxx/libsigrokcxx.hpp>
+
 #include "string.hpp"
 
 using std::string;
@@ -48,15 +51,18 @@ QWidget* String::get_widget(QWidget *parent, bool auto_commit)
 	if (!getter_)
 		return nullptr;
 
-	Glib::VariantBase variant = getter_();
-	if (!variant.gobj())
+	try {
+		Glib::VariantBase variant = getter_();
+		if (!variant.gobj())
+			return nullptr;
+	} catch (const sigrok::Error &e) {
+		qWarning() << tr("Querying config key %1 resulted in %2").arg(name_, e.what());
 		return nullptr;
-
-	string value = Glib::VariantBase::cast_dynamic<Glib::Variant<ustring>>(
-		variant).get();
+	}
 
 	line_edit_ = new QLineEdit(parent);
-	line_edit_->setText(QString::fromStdString(value));
+
+	update_widget();
 
 	if (auto_commit)
 		connect(line_edit_, SIGNAL(textEdited(const QString&)),
@@ -65,6 +71,28 @@ QWidget* String::get_widget(QWidget *parent, bool auto_commit)
 	return line_edit_;
 }
 
+void String::update_widget()
+{
+	if (!line_edit_)
+		return;
+
+	Glib::VariantBase variant;
+
+	try {
+		variant = getter_();
+	} catch (const sigrok::Error &e) {
+		qWarning() << tr("Querying config key %1 resulted in %2").arg(name_, e.what());
+		return;
+	}
+
+	assert(variant.gobj());
+
+	string value = Glib::VariantBase::cast_dynamic<Glib::Variant<ustring>>(
+		variant).get();
+
+	line_edit_->setText(QString::fromStdString(value));
+}
+
 void String::commit()
 {
 	assert(setter_);
