@@ -20,6 +20,7 @@
 #ifndef PULSEVIEW_PV_VIEWS_TRACEVIEW_DECODETRACE_HPP
 #define PULSEVIEW_PV_VIEWS_TRACEVIEW_DECODETRACE_HPP
 
+#include <config.h>
 #include "trace.hpp"
 
 #include <list>
@@ -29,20 +30,34 @@
 
 #include <QColor>
 #include <QComboBox>
+#include <QCheckBox>
+#include <QElapsedTimer>
+#include <QPolygon>
+#include <QPushButton>
 #include <QSignalMapper>
 #include <QTimer>
 
 #include <pv/binding/decoder.hpp>
+#include <pv/data/decode/decoder.hpp>
 #include <pv/data/decode/annotation.hpp>
 #include <pv/data/decode/row.hpp>
 #include <pv/data/signalbase.hpp>
 
+#define DECODETRACE_SHOW_RENDER_TIME 0
+
+using std::deque;
 using std::list;
 using std::map;
+using std::mutex;
 using std::pair;
 using std::shared_ptr;
 using std::vector;
 
+using pv::data::SignalBase;
+using pv::data::decode::Annotation;
+using pv::data::decode::Decoder;
+using pv::data::decode::Row;
+
 struct srd_channel;
 struct srd_decoder;
 
@@ -56,6 +71,7 @@ class DecodeSignal;
 
 namespace decode {
 class Decoder;
+class Row;
 }
 }  // namespace data
 
@@ -66,6 +82,43 @@ class DecoderGroupBox;
 namespace views {
 namespace trace {
 
+class ContainerWidget;
+
+struct DecodeTraceRow {
+	// When adding a field, make sure it's initialized properly in
+	// DecodeTrace::update_rows()
+
+	Row* decode_row;
+	unsigned int height, expanded_height, title_width, animation_step;
+	bool exists, currently_visible, has_hidden_classes;
+	bool expand_marker_highlighted, expanding, expanded, collapsing;
+	QPolygon expand_marker_shape;
+	float anim_height, anim_shape;
+
+	ContainerWidget* container;
+	QWidget* header_container;
+	QWidget* selector_container;
+	QCheckBox* row_visibility_checkbox;
+	vector<QCheckBox*> selectors;
+
+	QColor row_color;
+	map<uint32_t, QColor> ann_class_color;
+	map<uint32_t, QColor> ann_class_dark_color;
+};
+
+class ContainerWidget : public QWidget
+{
+	Q_OBJECT
+
+public:
+	ContainerWidget(QWidget *parent = nullptr);
+
+	virtual void resizeEvent(QResizeEvent* event);
+
+Q_SIGNALS:
+	void widgetResized(QWidget* sender);
+};
+
 class DecodeTrace : public Trace
 {
 	Q_OBJECT
@@ -73,6 +126,10 @@ class DecodeTrace : public Trace
 private:
 	static const QColor ErrorBgColor;
 	static const QColor NoDecodeColor;
+	static const QColor ExpandMarkerWarnColor;
+	static const QColor ExpandMarkerHiddenColor;
+	static const uint8_t ExpansionAreaHeaderAlpha;
+	static const uint8_t ExpansionAreaAlpha;
 
 	static const int ArrowSize;
 	static const double EndCapWidth;
@@ -80,14 +137,24 @@ private:
 	static const int DrawPadding;
 
 	static const int MaxTraceUpdateRate;
+	static const int AnimationDurationInTicks;
+	static const int HiddenRowHideDelay;
 
 public:
-	DecodeTrace(pv::Session &session, shared_ptr<data::SignalBase> signalbase,
+	DecodeTrace(pv::Session &session, shared_ptr<SignalBase> signalbase,
 		int index);
 
+	~DecodeTrace();
+
 	bool enabled() const;
 
-	shared_ptr<data::SignalBase> base() const;
+	shared_ptr<SignalBase> base() const;
+
+	/**
+	 * Sets the owner this trace in the view trace hierachy.
+	 * @param The new owner of the trace.
+	 */
+	virtual void set_owner(TraceTreeItemOwner *owner);
 
 	/**
 	 * Computes the vertical extents of the contents of this row item.
@@ -122,33 +189,30 @@ public:
 
 	virtual QMenu* create_view_context_menu(QWidget *parent, QPoint &click_pos);
 
-	void delete_pressed();
+	virtual void delete_pressed();
+
+	virtual void hover_point_changed(const QPoint &hp);
+
+	virtual void mouse_left_press_event(const QMouseEvent* event);
 
 private:
-	void draw_annotations(vector<pv::data::decode::Annotation> annotations,
-		QPainter &p, int h, const ViewItemPaintParams &pp, int y,
-		QColor row_color, int row_title_width);
+	void draw_annotations(deque<const Annotation*>& annotations, QPainter &p,
+		const ViewItemPaintParams &pp, int y, const DecodeTraceRow& row);
 
-	void draw_annotation(const pv::data::decode::Annotation &a, QPainter &p,
-		int h, const ViewItemPaintParams &pp, int y,
-		QColor row_color, int row_title_width) const;
+	void draw_annotation(const Annotation* a, QPainter &p,
+		const ViewItemPaintParams &pp, int y, const DecodeTraceRow& row) const;
 
-	void draw_annotation_block(qreal start, qreal end,
-		pv::data::decode::Annotation::Class ann_class, bool use_ann_format,
-		QPainter &p, int h, int y, QColor row_color) const;
+	void draw_annotation_block(qreal start, qreal end, Annotation::Class ann_class,
+		bool use_ann_format, QPainter &p, int y, const DecodeTraceRow& row) const;
 
-	void draw_instant(const pv::data::decode::Annotation &a, QPainter &p,
-		int h, qreal x, int y) const;
+	void draw_instant(const Annotation* a, QPainter &p, qreal x, int y) const;
 
-	void draw_range(const pv::data::decode::Annotation &a, QPainter &p,
-		int h, qreal start, qreal end, int y, const ViewItemPaintParams &pp,
-		int row_title_width) const;
+	void draw_range(const Annotation* a, QPainter &p, qreal start, qreal end,
+		int y, const ViewItemPaintParams &pp, int row_title_width) const;
 
-	void draw_error(QPainter &p, const QString &message,
-		const ViewItemPaintParams &pp);
+	void draw_error(QPainter &p, const QString &message, const ViewItemPaintParams &pp);
 
-	void draw_unresolved_period(QPainter &p, int h, int left,
-		int right) const;
+	void draw_unresolved_period(QPainter &p, int left, int right) const;
 
 	pair<double, double> get_pixels_offset_samples_per_pixel() const;
 
@@ -164,25 +228,42 @@ private:
 	QColor get_row_color(int row_index) const;
 	QColor get_annotation_color(QColor row_color, int annotation_index) const;
 
-	int get_row_at_point(const QPoint &point);
+	unsigned int get_row_y(const DecodeTraceRow* row) const;
+
+	DecodeTraceRow* get_row_at_point(const QPoint &point);
 
 	const QString get_annotation_at_point(const QPoint &point);
 
-	void create_decoder_form(int index,
-		shared_ptr<pv::data::decode::Decoder> &dec,
+	void update_stack_button();
+
+	void create_decoder_form(int index, shared_ptr<Decoder> &dec,
 		QWidget *parent, QFormLayout *form);
 
 	QComboBox* create_channel_selector(QWidget *parent,
-		const data::DecodeChannel *ch);
+		const data::decode::DecodeChannel *ch);
 	QComboBox* create_channel_selector_init_state(QWidget *parent,
-		const data::DecodeChannel *ch);
+		const data::decode::DecodeChannel *ch);
 
-	void export_annotations(vector<data::decode::Annotation> *annotations) const;
+	void export_annotations(deque<const Annotation*>& annotations) const;
 
-public:
-	virtual void hover_point_changed(const QPoint &hp);
+	void initialize_row_widgets(DecodeTraceRow* r, unsigned int row_id);
+	void update_rows();
+
+	/**
+	 * Sets row r to expanded state without forcing an update of the view
+	 */
+	void set_row_expanded(DecodeTraceRow* r);
+
+	/**
+	 * Sets row r to collapsed state without forcing an update of the view
+	 */
+	void set_row_collapsed(DecodeTraceRow* r);
+
+	void update_expanded_rows();
 
 private Q_SLOTS:
+	void on_setting_changed(const QString &key, const QVariant &value);
+
 	void on_new_annotations();
 	void on_delayed_trace_update();
 	void on_decode_reset();
@@ -202,6 +283,13 @@ private Q_SLOTS:
 	void on_delete_decoder(int index);
 
 	void on_show_hide_decoder(int index);
+	void on_show_hide_row(int row_id);
+	void on_show_hide_class(QWidget* sender);
+	void on_show_all_classes();
+	void on_hide_all_classes();
+	void on_row_container_resized(QWidget* sender);
+
+	void on_copy_annotation_to_clipboard();
 
 	void on_export_row();
 	void on_export_all_rows();
@@ -210,29 +298,42 @@ private Q_SLOTS:
 	void on_export_row_from_here();
 	void on_export_all_rows_from_here();
 
+	void on_animation_timer();
+	void on_hide_hidden_rows();
+
 private:
 	pv::Session &session_;
 	shared_ptr<data::DecodeSignal> decode_signal_;
 
-	vector<data::decode::Row> visible_rows_;
+	deque<DecodeTraceRow> rows_;
+	mutable mutex row_modification_mutex_;
 
 	map<QComboBox*, uint16_t> channel_id_map_;  // channel selector -> decode channel ID
 	map<QComboBox*, uint16_t> init_state_map_;  // init state selector -> decode channel ID
 	list< shared_ptr<pv::binding::Decoder> > bindings_;
 
-	data::decode::Row *selected_row_;
+	const Row* selected_row_;
 	pair<uint64_t, uint64_t> selected_sample_range_;
 
 	vector<pv::widgets::DecoderGroupBox*> decoder_forms_;
+	QPushButton* stack_button_;
 
-	map<data::decode::Row, int> row_title_widths_;
-	int row_height_, max_visible_rows_;
+	unsigned int default_row_height_, annotation_height_;
+	unsigned int visible_rows_;
 
 	int min_useful_label_width_;
+	bool always_show_all_rows_, show_hidden_rows_;
 
 	QSignalMapper delete_mapper_, show_hide_mapper_;
+	QSignalMapper row_show_hide_mapper_, class_show_hide_mapper_;
 
-	QTimer delayed_trace_updater_;
+	QTimer delayed_trace_updater_, animation_timer_, delayed_hidden_row_hider_;
+
+	QPolygon default_marker_shape_;
+
+#if DECODETRACE_SHOW_RENDER_TIME
+	QElapsedTimer render_time_;
+#endif
 };
 
 } // namespace trace
