@@ -25,6 +25,8 @@
 #include <QPoint>
 #include <QWidget>
 
+#include <pv/util.hpp>
+
 using std::shared_ptr;
 using std::vector;
 
@@ -131,6 +133,9 @@ protected:
 	void mouseReleaseEvent(QMouseEvent *event);
 	void mouseMoveEvent(QMouseEvent *event);
 
+	void keyPressEvent(QKeyEvent *event);
+	void keyReleaseEvent(QKeyEvent *event);
+
 	void leaveEvent(QEvent *event);
 
 public Q_SLOTS:
@@ -143,7 +148,12 @@ protected:
 	pv::views::trace::View &view_;
 	QPoint mouse_point_;
 	QPoint mouse_down_point_;
+	pv::util::Timestamp mouse_down_offset_;
 	shared_ptr<ViewItem> mouse_down_item_;
+
+	/// Keyboard modifiers that were active when mouse was last moved or clicked
+	Qt::KeyboardModifiers mouse_modifiers_;
+
 	bool item_dragging_;
 };
 
